@@ -2,15 +2,16 @@
 #include <fcntl.h>
 #include <string.h>
 #include <errno.h>
-#include <linux/zaptel.h>
+#include "kernel/zaptel.h"
 #include <stdio.h>
+#include <linux/types.h>
 #include <linux/ppp_defs.h> 
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include "bittest.h"
 #define FAST_HDLC_NEED_TABLES
-#include "fasthdlc.h"
+#include "kernel/fasthdlc.h"
 
 /* #define BLOCK_SIZE 2048 */
 #define BLOCK_SIZE 2041
@@ -74,7 +75,6 @@ void send_packet(unsigned char *buf, int len)
 {
 	int res;
 	int x;
-	int out;
 	unsigned char outbuf[BLOCK_SIZE];
 	int pos=0;
 	unsigned int fcs = PPP_INITFCS;
@@ -112,17 +112,12 @@ void send_packet(unsigned char *buf, int len)
 
 int main(int argc, char *argv[])
 {
-	int fd2;
-	int ires, res, x;
-	int i;
+	int res, x;
 	ZT_PARAMS tp;
 	ZT_BUFFERINFO bi;
 	int bs = BLOCK_SIZE;
 	unsigned char c=0;
-	unsigned char inbuf[BLOCK_SIZE];
 	unsigned char outbuf[BLOCK_SIZE];
-	unsigned int fcs;
-	static int packets=0;
 	if (argc < 2) {
 		fprintf(stderr, "Usage: markhdlctest <tor device>\n");
 		exit(1);
