@@ -8,7 +8,7 @@
 #include <usb.h>
 #include "hexfile.h"
 
-static const char rcsid[] = "$Id: fpga_load.c 3793 2008-02-04 23:00:48Z tzafrir $";
+static const char rcsid[] = "$Id: fpga_load.c 4266 2008-05-13 21:08:09Z tzafrir $";
 
 #define	ERR(fmt, arg...) do { \
 				if(verbose >= LOG_ERR) \
@@ -574,6 +574,10 @@ int my_usb_device_init(const char devpath[], struct my_usb_device *mydev, const
 	}
 	dev_desc = &mydev->dev->descriptor;
 	config_desc = mydev->dev->config;
+	if (!config_desc) {
+		ERR("usb interface without a configuration\n");
+		return 0;
+	}
 	interface = &config_desc->interface[abtype->my_interface_num];
 	iface_desc = interface->altsetting;
 	endpoint = iface_desc->endpoint;
