@@ -1,11 +1,10 @@
 #include <stdio.h>
 
-#include "fxo_modes.h"
+#include "wctdm_fxomodes.h"
 
 int main() {
 	size_t i;
 	
-	printf("case \"$mode\" in\n");
 	for (i=0; i<(sizeof(fxo_modes)/sizeof(struct fxo_mode)); i++) {
 		if (fxo_modes[i].name == NULL) break;
 		int reg16=0, reg26=0, reg30=0, reg31=0x20;
@@ -23,22 +22,12 @@ int main() {
 		
 		reg31 |= (fxo_modes[i].ohs2 << 3);
 
-		if (fxo_modes[i].ring_osc !=0 ) {
-			snprintf(ring_osc, BUFSIZ, "; ring_osc=\"%02X %02X\"",
-				(fxo_modes[i].ring_osc)>>8,
-				(fxo_modes[i].ring_osc)&&0xFF
-			);
-		}
-		if (fxo_modes[i].ring_x !=0 ) {
-			snprintf(ring_x, BUFSIZ, "; ring_x=\"%02X %02X\"",
-				(fxo_modes[i].ring_x)>>8,
-				(fxo_modes[i].ring_x)&&0xFF
-			);
-		}
-		
-		printf("%s)\treg16=%02X; reg26=%02X; reg30=%02X; reg31=%02X%s%s;;\n",
+		if (fxo_modes[i].ring_osc)
+			snprintf(ring_osc, BUFSIZ, "ring_osc=%04X", fxo_modes[i].ring_osc);
+		if (fxo_modes[i].ring_x)
+			snprintf(ring_x, BUFSIZ, "ring_x=%04X", fxo_modes[i].ring_x);
+		printf("%-15s\treg16=%02X\treg26=%02X\treg30=%02X\treg31=%02X\t%s\t%s\n",
 		       fxo_modes[i].name, reg16, reg26, reg30, reg31, ring_osc, ring_x);
 	}
-	printf("esac\n");
 	return 0;
 }
