@@ -5,7 +5,7 @@ package Zaptel::Config::Defaults;
 # This program is free software; you can redistribute and/or
 # modify it under the same terms as Perl itself.
 #
-# $Id: Defaults.pm 3793 2008-02-04 23:00:48Z tzafrir $
+# $Id: Defaults.pm 4187 2008-04-21 21:27:59Z tzafrir $
 #
 use strict;
 
@@ -46,7 +46,9 @@ sub source_vars {
 			}
 		}
 	}
-	die "No default_file" unless $default_file;
+	if (! $default_file) {
+		return ("", ());
+	}
 	my %vars = Zaptel::Config::Defaults::do_source($default_file, @vars);
 	return ($default_file, %vars);
 }
