@@ -43,12 +43,12 @@
 #endif
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_usb.c 3957 2008-03-07 00:45:53Z tzafrir $";
+static const char rcsid[] = "$Id: xpp_usb.c 4266 2008-05-13 21:08:09Z tzafrir $";
 
-DEF_PARM(int, print_dbg, 0, 0644, "Print DBG statements");	/* must be before zap_debug.h */
-DEF_PARM(int, usb1, 0, 0644, "Allow using USB 1.1 interfaces");
-DEF_PARM(uint, tx_sluggish, 2000, 0644, "A sluggish transmit (usec)");
-DEF_PARM(uint, drop_pcm_after, 6, 0644, "Number of consecutive tx_sluggish to drop a PCM frame");
+static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before zap_debug.h */
+static DEF_PARM(int, usb1, 0, 0644, "Allow using USB 1.1 interfaces");
+static DEF_PARM(uint, tx_sluggish, 2000, 0644, "A sluggish transmit (usec)");
+static DEF_PARM(uint, drop_pcm_after, 6, 0644, "Number of consecutive tx_sluggish to drop a PCM frame");
 
 #include "zap_debug.h"
 
@@ -58,7 +58,7 @@ DEF_PARM(uint, drop_pcm_after, 6, 0644, "Number of consecutive tx_sluggish to dr
 		THIS_MODULE->name, (xusb)->index, xusb->path, xusb->serial, ## __VA_ARGS__)
 
 #define	XUSB_DBG(bits, xusb, fmt, ...)	\
-			((void)((print_dbg & (DBG_ ## bits)) && XUSB_PRINTK(DEBUG, xusb, "%s: " fmt, __FUNCTION__, ## __VA_ARGS__)))
+			((void)((debug & (DBG_ ## bits)) && XUSB_PRINTK(DEBUG, xusb, "%s: " fmt, __FUNCTION__, ## __VA_ARGS__)))
 #define	XUSB_ERR(xusb, fmt, ...)		XUSB_PRINTK(ERR, xusb, fmt, ## __VA_ARGS__)
 #define	XUSB_NOTICE(xusb, fmt, ...)		XUSB_PRINTK(NOTICE, xusb, fmt, ## __VA_ARGS__)
 #define	XUSB_INFO(xusb, fmt, ...)		XUSB_PRINTK(INFO, xusb, fmt, ## __VA_ARGS__)
@@ -323,7 +323,7 @@ static xframe_t *alloc_xframe(xbus_t *xbus, gfp_t gfp_flags)
 	return &uframe->xframe;
 }
 
-void free_xframe(xbus_t *xbus, xframe_t *xframe)
+static void free_xframe(xbus_t *xbus, xframe_t *xframe)
 {
 	struct uframe	*uframe = xframe_to_uframe(xframe);
 	struct urb	*urb = &uframe->urb;
@@ -348,13 +348,14 @@ static int do_send_xframe(xbus_t *xbus, xframe_t *xframe)
 	xusb_t			*xusb;
 	int			ret = 0;
 	struct uframe		*uframe;
-	static int		rate_limit;
 
 	BUG_ON(!xframe);
 	BUG_ON(xframe->xframe_magic != XFRAME_MAGIC);
 	xusb = xusb_of(xbus);
 	BUG_ON(!xusb);
 	if(!xusb->present) {
+		static int	rate_limit;
+
 		if((rate_limit++ % 1003) == 0)
 			XUSB_ERR(xusb,
 				"abort do_send_xframe during device disconnect (%d)\n", rate_limit);
@@ -393,8 +394,8 @@ static int do_send_xframe(xbus_t *xbus, xframe_t *xframe)
 		ret = -EBADF;
 		goto failure;
 	}
-//	if (print_dbg)
-//		dump_xframe("USB_FRAME_SEND", xbus, xframe, print_dbg);
+//	if (debug)
+//		dump_xframe("USB_FRAME_SEND", xbus, xframe, debug);
 	atomic_inc(&xusb->pending_writes);
 	return 0;
 failure:
@@ -480,7 +481,7 @@ static const struct xusb_model_info {
 	struct xusb_endpoint	in;
 	struct xusb_endpoint	out;
 } model_table[] = {
-	[MODEL_FPGA_XPD] {
+	[MODEL_FPGA_XPD] = {
 		.iface_num = 0,
 		.in = { .ep_addr = 0x86 },
 		.out = { .ep_addr = 0x02 },
@@ -950,8 +951,8 @@ static void xpp_receive_callback(USB_PASS_CB(urb))
 	atomic_set(&xframe->frame_len, size);
 	xframe->tv_received = now;
 
-//	if (print_dbg)
-//		dump_xframe("USB_FRAME_RECEIVE", xbus, xframe, print_dbg);
+//	if (debug)
+//		dump_xframe("USB_FRAME_RECEIVE", xbus, xframe, debug);
 	XUSB_COUNTER(xusb, RX_FRAMES)++;
 	/* Send UP */
 	xbus_receive_xframe(xbus, xframe);
@@ -978,7 +979,7 @@ static void xpp_usb_cleanup(void)
 	}
 }
 
-int __init xpp_usb_init(void)
+static int __init xpp_usb_init(void)
 {
 	int	ret;
 	//xusb_t *xusb;
@@ -1009,7 +1010,7 @@ failure:
 }
 
 
-void __exit xpp_usb_shutdown(void)
+static void __exit xpp_usb_shutdown(void)
 {
 	DBG(GENERAL, "\n");
 	/* deregister this driver with the USB subsystem */
