@@ -433,7 +433,7 @@ static void dynamic_destroy(struct zt_dynamic *z)
 		kfree(z->msgbuf);
 
 	/* Free channels */
-	if (z->chans);
+	if (z->chans)
 		vfree(z->chans);
 	/* Free z */
 	kfree(z);
