@@ -11,7 +11,7 @@
 #include "bittest.h"
 
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #else
 #include <zaptel/zaptel.h>
 #endif
@@ -37,7 +37,7 @@ int main(int argc, char *argv[])
 	unsigned char c=0;
 	unsigned char outbuf[BLOCK_SIZE];
 	if (argc < 2) {
-		fprintf(stderr, "Usage: markhdlctest <tor device>\n");
+		fprintf(stderr, "Usage: %s <tor device>\n", argv[0]);
 		exit(1);
 	}
 	fd = open(argv[1], O_RDWR, 0600);
