@@ -3,15 +3,13 @@
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <linux/types.h>
-#include <linux/ppp_defs.h> 
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <time.h>
 
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #else
 #include <zaptel/zaptel.h>
 #endif
@@ -44,7 +42,7 @@ int main(int argc, char *argv[])
 	int timeout=0;
 	time_t start_time=0;
 	if (argc < 2 || argc > 3 ) {
-		fprintf(stderr, "Usage: %s <zaptel device (optional timeout)>\n",argv[0]);
+		fprintf(stderr, "Usage: %s <zaptel device> [timeout]\n",argv[0]);
 		exit(1);
 	}
 	fd = open(argv[1], O_RDWR, 0600);
