@@ -9,7 +9,7 @@
 #include <errno.h>
 
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #else
 #include <zaptel/zaptel.h>
 #endif
