@@ -25,7 +25,7 @@
 #define _TONEZONE_H
 
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #else
 #include <zaptel/zaptel.h>
 #endif
@@ -41,11 +41,17 @@ struct tone_zone_sound {
 };
 
 struct tone_zone {
-	int zone;					/* Zone number */
+	int zone;				/* Zone number */
 	char country[10];			/* Country code */
-	char description[40];		/* Description */
+	char description[40];			/* Description */
 	int ringcadence[ZT_MAX_CADENCE];	/* Ring cadence */
 	struct tone_zone_sound tones[ZT_TONE_MAX];
+	int dtmf_high_level;			/* Power level of high frequency component
+						   of DTMF, expressed in dBm0. */
+	int dtmf_low_level;			/* Power level of low frequency component
+						   of DTMF, expressed in dBm0. */
+	int mfr1_level;				/* Power level of MFR1, expressed in dBm0. */
+	int mfr2_level;				/* Power level of MFR2, expressed in dBm0. */
 };
 
 extern struct tone_zone builtin_zones[];
