@@ -6,7 +6,7 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include "zap.h"
-#include <linux/zaptel.h>
+#include <zaptel.h>
 
 int main(int argc, char *argv[])
 {
