@@ -160,6 +160,8 @@ static const char *sigtype_to_str(const int sig)
 		return "HDLC with FCS check";
 	case ZT_SIG_HARDHDLC:
 		return "Hardware assisted D-channel";
+	case ZT_SIG_MTP2:
+		return "MTP2";
 	default:
 		return "Unknown";
 	}
@@ -590,6 +592,9 @@ static int chanconfig(char *keyword, char *args)
 			} else if (!strcasecmp(keyword, "hardhdlc")) {
 				sig[x] = "Hardware assisted D-channel";
 				cc[x].sigtype = ZT_SIG_HARDHDLC;
+			} else if (!strcasecmp(keyword, "mtp2")) {
+				sig[x] = "MTP2";
+				cc[x].sigtype = ZT_SIG_MTP2;
 			} else {
 				fprintf(stderr, "Huh? (%s)\n", keyword);
 			}
@@ -1018,6 +1023,7 @@ static int rad_chanconfig(char *keyword, char *args)
 	int x,i,n;
 	struct zt_radio_param p;
 
+	toneindex = 1;
 	bzero(chans, sizeof(chans));
 	res = rad_apply_channels(chans, args);
 	if (res <= 0)
@@ -1200,6 +1206,7 @@ static struct handler {
 	{ "nethdlc", chanconfig },
 	{ "fcshdlc", chanconfig },
 	{ "hardhdlc", chanconfig },
+	{ "mtp2", chanconfig },
 	{ "dchan", chanconfig },
 	{ "bchan", chanconfig },
 	{ "indclear", chanconfig },
